@@ -34,28 +34,7 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #else
 #define obj_symbol_slew(bfd) 0
 #endif
-#define obj_string_table(bfd)	(coff_data(bfd)->string_table)
 
-#if 0
-typedef struct coff_ptr_struct
-{
-  unsigned int offset;
-  char fix_tag;
-  char fix_end;
-  union {
-    union internal_auxent auxent;
-    struct internal_syment syment;
-  } u;
-} combined_entry_type;
-
-
-typedef struct 
-{
-  asymbol symbol;
-  combined_entry_type *native;
-  struct lineno_cache_entry *lineno;
-} coff_symbol_type;
-#endif
 
 /* `Tdata' information kept for COFF files.  */
 
@@ -71,7 +50,6 @@ typedef struct coff_tdata
   struct coff_ptr_struct *raw_syments;
   struct lineno *raw_linenos;
   unsigned int raw_syment_count;
-  char *string_table;
   unsigned short flags;
 
   /* These are only valid once writing has begun */
@@ -95,76 +73,21 @@ typedef struct coff_tdata
 
 
 
-/*THE FOLLOWING IS EXTRACTED FROM THE SOURCE*/
-
-/*:coffcode.h*/
-
-/*
-The hidden information for an asymbol is:
-*/
-
- typedef struct coff_ptr_struct
- {
-
-/*
-Remembers the offset from the first symbol in the file for this
-symbol. Generated by @code{coff_renumber_symbols}.
-*/
-
-   unsigned int offset;
-
-/*
-Should the tag field of this symbol be renumbered.
-Created by @code{coff_pointerize_aux}.
-*/
-
-   char fix_tag;
-
-/*
-Should the endidx field of this symbol be renumbered.
-Created by @code{coff_pointerize_aux}.
-*/
-
-   char fix_end;
-
-/*
-The container for the symbol structure as read and translated from the file.
-*/
-
-   union {
-     union internal_auxent auxent;
-     struct internal_syment syment;
-   } u;
- } combined_entry_type;
-
-/*
-
-Each canonical asymbol really looks like this:
-*/
-
- typedef struct coff_symbol_struct
- {
-
-/*
-The actual symbol which the rest of BFD works with
-*/
-
-   asymbol symbol;
-
-/*
-A pointer to the hidden information for this symbol
-*/
-
-   combined_entry_type *native;
-
-/*
-A pointer to the linenumber information for this symbol
-*/
-
-   struct lineno_cache_entry *lineno;
- } coff_symbol_type;
-
-/*
-*/
-
+/* And more taken from the source .. */
 
+typedef struct coff_ptr_struct 
+{
+unsigned int offset;
+char fix_tag;
+char fix_end;
+union {
+   union internal_auxent auxent;
+   struct internal_syment syment;
+ } u;
+} combined_entry_type;
+typedef struct coff_symbol_struct
+{
+asymbol symbol;
+combined_entry_type *native;
+struct lineno_cache_entry *lineno;
+} coff_symbol_type;
