@@ -19,7 +19,6 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #include <stdio.h>
 #include "defs.h"
-#include "param.h"
 #include "frame.h"
 #include "inferior.h"
 #include "language.h"
@@ -42,6 +41,8 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include <sys/reg.h>
 #include "ieee-float.h"
 
+#include "target.h"
+
 extern void print_387_control_word ();		/* i387-tdep.h */
 extern void print_387_status_word ();
 
@@ -81,6 +82,9 @@ i386_register_u_addr (blockend, regnum)
   
 }
 
+#if 0
+/* mauro@olympus 1991.10.20 -- compiling the following code causes
+   undefined symbols at link time, specifically: corechan, have_inferior_p */
 struct env387 
 {
   unsigned short control;
@@ -177,14 +181,14 @@ i386_float_info ()
   struct fpstate *fpstatep;
   char buf[sizeof (struct fpstate) + 2 * sizeof (int)];
   unsigned int uaddr;
-  char fpvalid;
+  char fpvalid = 0;
   unsigned int rounded_addr;
   unsigned int rounded_size;
   extern int corechan;
   int skip;
   
   uaddr = (char *)&u.u_fpvalid - (char *)&u;
-  if (have_inferior_p()) 
+  if (target_has_execution)
     {
       unsigned int data;
       unsigned int mask;
@@ -195,6 +199,7 @@ i386_float_info ()
       
       fpvalid = ((data & mask) != 0);
     } 
+#if 0
   else 
     {
       if (lseek (corechan, uaddr, 0) < 0)
@@ -203,6 +208,7 @@ i386_float_info ()
 	perror ("read on core file");
       
     }
+#endif	/* no core support yet */
   
   if (fpvalid == 0) 
     {
@@ -211,7 +217,7 @@ i386_float_info ()
     }
   
   uaddr = (char *)&U_FPSTATE(u) - (char *)&u;
-  if (have_inferior_p ()) 
+  if (target_has_execution)
     {
       int *ip;
       
@@ -227,6 +233,7 @@ i386_float_info ()
 	  rounded_addr += sizeof (int);
 	}
     } 
+#if 0
   else 
     {
       if (lseek (corechan, uaddr, 0) < 0)
@@ -235,7 +242,9 @@ i386_float_info ()
 	perror_with_name ("read from core file");
       skip = 0;
     }
-  
+ #endif	/* 0 */ 
+
   fpstatep = (struct fpstate *)(buf + skip);
   print_387_status (fpstatep->status, (struct env387 *)fpstatep->state);
 }
+#endif /* mauro@olympus 1991.10.20 */
