@@ -41,6 +41,8 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include "target.h"
 #include "parser-defs.h"
 
+extern volatile void return_to_top_level ();
+
 /* Forward function declarations */
 static void set_type_range ();
 
@@ -122,6 +124,7 @@ set_language_command (ignore, from_tty)
     printf("The currently understood settings are:\n\n\
 local or auto    Automatic setting based on source file\n\
 c                Use the C language\n\
+c++              Use the C++ language\n\
 modula-2         Use the Modula-2 language\n");
     /* Restore the silly string. */
     set_language(current_language->la_language);
@@ -280,6 +283,7 @@ set_language(lang)
       current_language = languages[i];
       set_type_range ();
       set_lang_str();
+      break;
     }
   }
 }
@@ -295,7 +299,7 @@ set_lang_str()
    if (language_mode == language_mode_auto)
       prefix = "auto; currently ";
 
-   language = concat(prefix, current_language->la_name, "");
+   language = concat(prefix, current_language->la_name, NULL);
 }
 
 void
@@ -322,7 +326,7 @@ set_type_str()
       error ("Unrecognized type check setting.");
    }
 
-   type = concat(prefix,tmp,"");
+   type = concat(prefix,tmp,NULL);
 }
 
 void
@@ -349,7 +353,7 @@ set_range_str()
       error ("Unrecognized range check setting.");
    }
 
-   range = concat(pref,tmp,"");
+   range = concat(pref,tmp,NULL);
 }
 
 
@@ -379,6 +383,7 @@ binop_result_type(v1,v2)
    switch(current_language->la_language)
    {
    case language_c:
+   case language_cplus:
       if (TYPE_CODE(VALUE_TYPE(v1))==TYPE_CODE_FLT)
 	 return TYPE_CODE(VALUE_TYPE(v2)) == TYPE_CODE_FLT && l2 > l1 ?
 	    VALUE_TYPE(v2) : VALUE_TYPE(v1);
@@ -537,6 +542,7 @@ integral_type (type)
    switch(current_language->la_language)
    {
    case language_c:
+   case language_cplus:
       return (TYPE_CODE(type) != TYPE_CODE_INT) &&
 	 (TYPE_CODE(type) != TYPE_CODE_ENUM) ? 0 : 1;
    case language_m2:
@@ -572,6 +578,7 @@ character_type (type)
       return TYPE_CODE(type) != TYPE_CODE_CHAR ? 0 : 1;
 
    case language_c:
+   case language_cplus:
       return (TYPE_CODE(type) == TYPE_CODE_INT) &&
 	 TYPE_LENGTH(type) == sizeof(char)
 	 ? 1 : 0;
@@ -589,6 +596,7 @@ boolean_type (type)
       return TYPE_CODE(type) != TYPE_CODE_BOOL ? 0 : 1;
 
    case language_c:
+   case language_cplus:
       return TYPE_CODE(type) != TYPE_CODE_INT ? 0 : 1;
    }
 }
@@ -618,6 +626,7 @@ structured_type(type)
    switch(current_language->la_language)
    {
    case language_c:
+   case language_cplus:
       return (TYPE_CODE(type) == TYPE_CODE_STRUCT) ||
 	 (TYPE_CODE(type) == TYPE_CODE_UNION) ||
 	    (TYPE_CODE(type) == TYPE_CODE_ARRAY);
@@ -643,6 +652,7 @@ value_true(val)
   switch (current_language->la_language) {
 
   case language_c:
+  case language_cplus:
     return !value_zerop (val);
 
   case language_m2:
@@ -798,6 +808,7 @@ binop_type_check(arg1,arg2,op)
       {
 #ifdef _LANG_c
       case language_c:
+      case language_cplus:
 	 switch(op)
 	 {
 	 case BINOP_DIV:
@@ -963,18 +974,6 @@ add_language (lang)
 	languages_allocsize * sizeof (*languages));
     }
   languages[languages_size++] = lang;
-
-#if FIXME
-  if (targetlist == NULL)
-    add_prefix_cmd ("target", class_run, target_command,
-		    "Connect to a target machine or process.\n\
-The first argument is the type or protocol of the target machine.\n\
-Remaining arguments are interpreted by the target protocol.  For more\n\
-information on the arguments for a particular protocol, type\n\
-`help target ' followed by the protocol name.",
-		    &targetlist, "target ", 0, &cmdlist);
-  add_cmd (t->to_shortname, no_class, t->to_open, t->to_doc, &targetlist);
-#endif FIXME
 }
 
 /* Define the language that is no language.  */
