@@ -1,8 +1,5 @@
-/* Work with core dump and executable files, for GDB on MIPS. 
-   This code would be in core.c if it weren't machine-dependent. */
-
-/* Low level interface to ptrace, for GDB when running under Unix.
-   Copyright (C) 1988, 1989, 1990  Free Software Foundation, Inc.
+/* Target-dependent code for the MIPS architecture, for GDB, the GNU Debugger.
+   Copyright 1988, 1989, 1990, 1991  Free Software Foundation, Inc.
    Contributed by Alessandro Forin(af@cs.cmu.edu) at CMU
    and by Per Bothner(bothner@cs.wisc.edu) at U.Wisconsin.
 
@@ -24,7 +21,6 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #include <stdio.h>
 #include "defs.h"
-#include "param.h"
 #include "frame.h"
 #include "inferior.h"
 #include "symtab.h"
@@ -355,7 +351,7 @@ init_extra_frame_info(fci)
 	  for (ireg = 31; mask; --ireg, mask <<= 1)
 	      if (mask & 0x80000000)
 	      {
-		  fci->saved_regs->regs[32+ireg] = reg_position;
+		  fci->saved_regs->regs[FP0_REGNUM+ireg] = reg_position;
 		  reg_position -= 4;
 	      }
       }
@@ -478,7 +474,7 @@ mips_push_dummy_frame()
   for (ireg = 32; --ireg >= 0; )
     if (PROC_FREG_MASK(proc_desc) & (1 << ireg))
       {
-	buffer = read_register (ireg);
+	buffer = read_register (ireg + FP0_REGNUM);
 	write_memory (save_address, &buffer, 4);
 	save_address -= 4;
       }
@@ -524,10 +520,10 @@ mips_pop_frame()
       if (PROC_REG_MASK(proc_desc) & (1 << regnum))
 	write_register (regnum,
 		  read_memory_integer (frame->saved_regs->regs[regnum], 4));
-    for (regnum = 64; --regnum >= 32; )
+    for (regnum = 32; --regnum >= 0; )
       if (PROC_FREG_MASK(proc_desc) & (1 << regnum))
-	write_register (regnum,
-		  read_memory_integer (frame->saved_regs->regs[regnum], 4));
+	write_register (regnum + FP0_REGNUM,
+		  read_memory_integer (frame->saved_regs->regs[regnum + FP0_REGNUM], 4));
   }
   write_register (SP_REGNUM, new_sp);
   flush_cached_frames ();
