@@ -20,7 +20,6 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include <stdio.h>
 #include "defs.h"
 #include "symtab.h"
-#include "param.h"
 #include "gdbcore.h"
 #include "frame.h"
 #include "target.h"
@@ -41,6 +40,7 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 extern char *getenv ();
 
 extern char *cplus_demangle ();
+extern char *cplus_mangle_opname ();
 extern struct value *value_of_this ();
 extern void break_command ();
 extern void select_source_symtab ();
@@ -83,7 +83,7 @@ lookup_symtab_1 (name)
     if (!strcmp (name, ps->filename))
       {
 	if (ps->readin)
-	  error ("Internal: readin pst found when no symtab found.");
+	  error ("Internal: readin pst for `%s' found when no symtab found.", name);
 	return PSYMTAB_TO_SYMTAB (ps);
       }
 
@@ -106,7 +106,7 @@ lookup_symtab_1 (name)
 	      && !strcmp (ps->filename + l - len, name))
 	    {
 	      if (ps->readin)
-		error ("Internal: readin pst found when no symtab found.");
+		error ("Internal: readin pst for `%s' found when no symtab found.", name);
 	      return PSYMTAB_TO_SYMTAB (ps);
 	    }
 	}
@@ -164,27 +164,27 @@ type_name_no_tag (type)
      register struct type *type;
 {
   register char *name = TYPE_NAME (type);
-  char *strchr ();
+
   if (name == 0)
     return 0;
 
-#if 0
   switch (TYPE_CODE (type))
     {
     case TYPE_CODE_STRUCT:
-      return name + 7;
+      if(!strncmp(name,"struct ",7))
+	return name + 7;
+      else return name;
     case TYPE_CODE_UNION:
+      if(!strncmp(name,"union ",6))
       return name + 6;
+      else return name;
     case TYPE_CODE_ENUM:
+      if(!strncmp(name,"enum ",5))
       return name + 5;
+      else return name;
+    default:
+      return name;
     }
-#endif
-
-  name = strchr (name, ' ');
-  if (name)
-    return name + 1;
-
-  return TYPE_NAME (type);
 }
 
 /* Added by Bryan Boreham, Kewill, Sun Sep 17 18:07:17 1989.
@@ -224,31 +224,54 @@ check_stub_type(type)
 
 /* Demangle a GDB method stub type.  */
 char *
-gdb_mangle_typename (type)
+gdb_mangle_name (type, i, j)
      struct type *type;
+     int i, j;
 {
-  static struct type *last_type;
-  static char *mangled_typename;
+  int mangled_name_len;
+  char *mangled_name;
+  struct fn_field *f = TYPE_FN_FIELDLIST1 (type, i);
+  struct fn_field *method = &f[j];
+  char *field_name = TYPE_FN_FIELDLIST_NAME (type, i);
 
-  if (type != last_type)
+  /* Need a new type prefix.  */
+  char *strchr ();
+  char *const_prefix = method->is_const ? "C" : "";
+  char *volatile_prefix = method->is_volatile ? "V" : "";
+  char *newname = type_name_no_tag (type);
+  char buf[20];
+  int len = strlen (newname);
+
+  sprintf (buf, "__%s%s%d", const_prefix, volatile_prefix, len);
+  mangled_name_len = (strlen (field_name)
+			  + strlen (buf) + len
+			  + strlen (TYPE_FN_FIELD_PHYSNAME (f, j))
+			  + 1);
+
+  /* Only needed for GNU-mangled names.  ANSI-mangled names
+     work with the normal mechanisms.  */
+  if (OPNAME_PREFIX_P (field_name))
     {
-      /* Need a new type prefix.  */
-      char *strchr ();
-      char *newname = type_name_no_tag (type);
-      char buf[20];
-      int len;
+      char *opname = cplus_mangle_opname (field_name + 3, 0);
+      if (opname == NULL)
+	error ("No mangling for \"%s\"", field_name);
+      mangled_name_len += strlen (opname);
+      mangled_name = (char *)xmalloc (mangled_name_len);
 
-      if (mangled_typename)
-	free (mangled_typename);
-
-      len = strlen (newname);
-      sprintf (buf, "__%d", len);
-      mangled_typename = (char *)xmalloc (strlen (buf) + len + 1);
-      strcpy (mangled_typename, buf);
-      strcat (mangled_typename, newname);
-      /* Now we have built "__#newname".  */
+      strncpy (mangled_name, field_name, 3);
+      mangled_name[3] = '\0';
+      strcat (mangled_name, opname);
     }
-  return mangled_typename;
+  else
+    {
+      mangled_name = (char *)xmalloc (mangled_name_len);
+      strcpy (mangled_name, TYPE_FN_FIELDLIST_NAME (type, i));
+    }
+  strcat (mangled_name, buf);
+  strcat (mangled_name, newname);
+  strcat (mangled_name, TYPE_FN_FIELD_PHYSNAME (f, j));
+
+  return mangled_name;
 }
 
 /* Lookup a primitive type named NAME. 
@@ -356,6 +379,31 @@ lookup_enum (name, block)
   return SYMBOL_TYPE (sym);
 }
 
+/* Lookup a template type named "template NAME<TYPE>",
+   visible in lexical block BLOCK.  */
+
+struct type *
+lookup_template_type (name, type, block)
+     char *name;
+     struct type *type;
+     struct block *block;
+{
+  struct symbol *sym ;
+  char *nam = (char*) alloca(strlen(name) + strlen(type->name) + 4);
+  strcpy(nam, name);
+  strcat(nam, "<");
+  strcat(nam, type->name);
+  strcat(nam, " >");	/* FIXME, extra space still introduced in gcc? */
+
+  sym = lookup_symbol (nam, block, VAR_NAMESPACE, 0, (struct symtab **)NULL);
+
+  if (sym == 0)
+    error ("No template type named %s.", name);
+  if (TYPE_CODE (SYMBOL_TYPE (sym)) != TYPE_CODE_STRUCT)
+    error ("This context has class, union or enum %s, not a struct.", name);
+  return SYMBOL_TYPE (sym);
+}
+
 /* Given a type TYPE, lookup the type of the component of type named
    NAME.  
    If NOERR is nonzero, return zero if NAME is not suitably defined.  */
@@ -368,7 +416,7 @@ lookup_struct_elt_type (type, name, noerr)
 {
   int i;
 
-  if (TYPE_CODE (type) != TYPE_CODE_STRUCT
+  if (   TYPE_CODE (type) != TYPE_CODE_STRUCT
       && TYPE_CODE (type) != TYPE_CODE_UNION)
     {
       target_terminal_ours ();
@@ -410,17 +458,14 @@ lookup_struct_elt_type (type, name, noerr)
 }
 
 /* Given a type TYPE, return a type of pointers to that type.
-   May need to construct such a type if this is the first use.
-
-   C++: use TYPE_MAIN_VARIANT and TYPE_CHAIN to keep pointer
-   to member types under control.  */
+   May need to construct such a type if this is the first use.  */
 
 struct type *
 lookup_pointer_type (type)
      struct type *type;
 {
   register struct type *ptype = TYPE_POINTER_TYPE (type);
-  if (ptype) return TYPE_MAIN_VARIANT (ptype);
+  if (ptype) return ptype;
 
   /* This is the first time anyone wanted a pointer to a TYPE.  */
   if (TYPE_FLAGS (type) & TYPE_FLAG_PERM)
@@ -430,7 +475,6 @@ lookup_pointer_type (type)
 					    sizeof (struct type));
 
   bzero (ptype, sizeof (struct type));
-  TYPE_MAIN_VARIANT (ptype) = ptype;
   TYPE_TARGET_TYPE (ptype) = type;
   TYPE_POINTER_TYPE (type) = ptype;
   /* New type is permanent if type pointed to is permanent.  */
@@ -449,7 +493,7 @@ lookup_reference_type (type)
      struct type *type;
 {
   register struct type *rtype = TYPE_REFERENCE_TYPE (type);
-  if (rtype) return TYPE_MAIN_VARIANT (rtype);
+  if (rtype) return rtype;
 
   /* This is the first time anyone wanted a pointer to a TYPE.  */
   if (TYPE_FLAGS (type) & TYPE_FLAG_PERM)
@@ -459,7 +503,6 @@ lookup_reference_type (type)
 					    sizeof (struct type));
 
   bzero (rtype, sizeof (struct type));
-  TYPE_MAIN_VARIANT (rtype) = rtype;
   TYPE_TARGET_TYPE (rtype) = type;
   TYPE_REFERENCE_TYPE (type) = rtype;
   /* New type is permanent if type pointed to is permanent.  */
@@ -481,209 +524,119 @@ struct type *
 lookup_member_type (type, domain)
      struct type *type, *domain;
 {
-  register struct type *mtype = TYPE_MAIN_VARIANT (type);
-  struct type *main_type;
-
-  main_type = mtype;
-  while (mtype)
-    {
-      if (TYPE_DOMAIN_TYPE (mtype) == domain)
-	return mtype;
-      mtype = TYPE_NEXT_VARIANT (mtype);
-    }
-
-  /* This is the first time anyone wanted this member type.  */
-  if (TYPE_FLAGS (type) & TYPE_FLAG_PERM)
-    mtype  = (struct type *) xmalloc (sizeof (struct type));
-  else
-    mtype  = (struct type *) obstack_alloc (symbol_obstack,
-					    sizeof (struct type));
-
-  bzero (mtype, sizeof (struct type));
-  if (main_type == 0)
-    main_type = mtype;
-  else
-    {
-      TYPE_NEXT_VARIANT (mtype) = TYPE_NEXT_VARIANT (main_type);
-      TYPE_NEXT_VARIANT (main_type) = mtype;
-    }
-  TYPE_MAIN_VARIANT (mtype) = main_type;
-  TYPE_TARGET_TYPE (mtype) = type;
-  TYPE_DOMAIN_TYPE (mtype) = domain;
-  /* New type is permanent if type pointed to is permanent.  */
-  if (TYPE_FLAGS (type) & TYPE_FLAG_PERM)
-    TYPE_FLAGS (mtype) |= TYPE_FLAG_PERM;
-
-  /* In practice, this is never used.  */
-  TYPE_LENGTH (mtype) = 1;
-  TYPE_CODE (mtype) = TYPE_CODE_MEMBER;
-
-#if 0
-  /* Now splice in the new member pointer type.  */
-  if (main_type)
-    {
-      /* This type was not "smashed".  */
-      TYPE_CHAIN (mtype) = TYPE_CHAIN (main_type);
-      TYPE_CHAIN (main_type) = mtype;
-    }
-#endif
+  register struct type *mtype;
 
+  mtype  = (struct type *) obstack_alloc (symbol_obstack,
+					  sizeof (struct type));
+  smash_to_member_type (mtype, domain, type);
   return mtype;
 }
 
-/* Allocate a stub method whose return type is
-   TYPE.  We will fill in arguments later.  This always
-   returns a fresh type.  If we unify this type with
-   an existing type later, the storage allocated
-   here can be freed.  */
+/* Allocate a stub method whose return type is TYPE.  
+   This apparently happens for speed of symbol reading, since parsing
+   out the arguments to the method is cpu-intensive, the way we are doing
+   it.  So, we will fill in arguments later.
+   This always returns a fresh type.   */
+
 struct type *
 allocate_stub_method (type)
      struct type *type;
 {
-  struct type *mtype = (struct type *)xmalloc (sizeof (struct type));
+  struct type *mtype = (struct type *) obstack_alloc (symbol_obstack,
+						      sizeof (struct type));
   bzero (mtype, sizeof (struct type));
-  TYPE_MAIN_VARIANT (mtype) = mtype;
   TYPE_TARGET_TYPE (mtype) = type;
+  /*  _DOMAIN_TYPE (mtype) = unknown yet */
+  /*  _ARG_TYPES (mtype) = unknown yet */
   TYPE_FLAGS (mtype) = TYPE_FLAG_STUB;
   TYPE_CODE (mtype) = TYPE_CODE_METHOD;
   TYPE_LENGTH (mtype) = 1;
   return mtype;
 }
 
-/* Lookup a method type belonging to class DOMAIN, returning type TYPE,
-   and taking a list of arguments ARGS.
-   If one is not found, allocate a new one.  */
+/* Ugly hack to convert method stubs into method types.
 
-struct type *
-lookup_method_type (domain, type, args)
-     struct type *domain, *type, **args;
-{
-  register struct type *mtype = TYPE_MAIN_VARIANT (type);
-  struct type *main_type;
-
-  main_type = mtype;
-  while (mtype)
-    {
-      if (TYPE_DOMAIN_TYPE (mtype) == domain)
-	{
-	  struct type **t1 = args;
-	  struct type **t2 = TYPE_ARG_TYPES (mtype);
-	  if (t2)
-	    {
-	      int i;
-	      for (i = 0; t1[i] != 0 && t1[i]->code != TYPE_CODE_VOID; i++)
-		if (t1[i] != t2[i])
-		  break;
-	      if (t1[i] == t2[i])
-		return mtype;
-	    }
-	}
-      mtype = TYPE_NEXT_VARIANT (mtype);
-    }
-
-  /* This is the first time anyone wanted this member type.  */
-  if (TYPE_FLAGS (type) & TYPE_FLAG_PERM)
-    mtype  = (struct type *) xmalloc (sizeof (struct type));
-  else
-    mtype  = (struct type *) obstack_alloc (symbol_obstack,
-					    sizeof (struct type));
-
-  bzero (mtype, sizeof (struct type));
-  if (main_type == 0)
-    main_type = mtype;
-  else
-    {
-      TYPE_NEXT_VARIANT (mtype) = TYPE_NEXT_VARIANT (main_type);
-      TYPE_NEXT_VARIANT (main_type) = mtype;
-    }
-  TYPE_MAIN_VARIANT (mtype) = main_type;
-  TYPE_TARGET_TYPE (mtype) = type;
-  TYPE_DOMAIN_TYPE (mtype) = domain;
-  TYPE_ARG_TYPES (mtype) = args;
-  /* New type is permanent if type pointed to is permanent.  */
-  if (TYPE_FLAGS (type) & TYPE_FLAG_PERM)
-    TYPE_FLAGS (mtype) |= TYPE_FLAG_PERM;
-
-  /* In practice, this is never used.  */
-  TYPE_LENGTH (mtype) = 1;
-  TYPE_CODE (mtype) = TYPE_CODE_METHOD;
-
-#if 0
-  /* Now splice in the new member pointer type.  */
-  if (main_type)
-    {
-      /* This type was not "smashed".  */
-      TYPE_CHAIN (mtype) = TYPE_CHAIN (main_type);
-      TYPE_CHAIN (main_type) = mtype;
-    }
-#endif
-
-  return mtype;
-}
-
-#if 0
-/* Given a type TYPE, return a type which has offset OFFSET,
-   via_virtual VIA_VIRTUAL, and via_public VIA_PUBLIC.
-   May need to construct such a type if none exists.  */
-struct type *
-lookup_basetype_type (type, offset, via_virtual, via_public)
+   He ain't kiddin'.  This demangles the name of the method into a string
+   including argument types, parses out each argument type, generates
+   a string casting a zero to that type, evaluates the string, and stuffs
+   the resulting type into an argtype vector!!!  Then it knows the type
+   of the whole function (including argument types for overloading),
+   which info used to be in the stab's but was removed to hack back
+   the space required for them.  */
+void
+check_stub_method (type, i, j)
      struct type *type;
-     int offset;
-     int via_virtual, via_public;
+     int i, j;
 {
-  register struct type *btype = TYPE_MAIN_VARIANT (type);
-  struct type *main_type;
+  extern char *gdb_mangle_name (), *strchr ();
+  struct fn_field *f;
+  char *mangled_name = gdb_mangle_name (type, i, j);
+  char *demangled_name = cplus_demangle (mangled_name, 0);
+  char *argtypetext, *p;
+  int depth = 0, argcount = 1;
+  struct type **argtypes;
+  struct type *mtype;
 
-  if (offset != 0)
+  /* Now, read in the parameters that define this type.  */
+  argtypetext = strchr (demangled_name, '(') + 1;
+  p = argtypetext;
+  while (*p)
     {
-      printf ("Internal error: type offset non-zero in lookup_basetype_type");
-      offset = 0;
+      if (*p == '(')
+	depth += 1;
+      else if (*p == ')')
+	depth -= 1;
+      else if (*p == ',' && depth == 0)
+	argcount += 1;
+
+      p += 1;
+    }
+  /* We need two more slots: one for the THIS pointer, and one for the
+     NULL [...] or void [end of arglist].  */
+  argtypes = (struct type **) obstack_alloc (symbol_obstack,
+				(argcount+2) * sizeof (struct type *));
+  p = argtypetext;
+  argtypes[0] = lookup_pointer_type (type);
+  argcount = 1;
+
+  if (*p != ')')			/* () means no args, skip while */
+    {
+      depth = 0;
+      while (*p)
+	{
+	  if (depth <= 0 && (*p == ',' || *p == ')'))
+	    {
+	      argtypes[argcount] =
+		  parse_and_eval_type (argtypetext, p - argtypetext);
+	      argcount += 1;
+	      argtypetext = p + 1;
+	    }
+
+	  if (*p == '(')
+	    depth += 1;
+	  else if (*p == ')')
+	    depth -= 1;
+
+	  p += 1;
+	}
     }
 
-  main_type = btype;
-  while (btype)
-    {
-      if (/* TYPE_OFFSET (btype) == offset
-	  && */ TYPE_VIA_PUBLIC (btype) == via_public
-	  && TYPE_VIA_VIRTUAL (btype) == via_virtual)
-	return btype;
-      btype = TYPE_NEXT_VARIANT (btype);
-    }
-
-  /* This is the first time anyone wanted this member type.  */
-  if (TYPE_FLAGS (type) & TYPE_FLAG_PERM)
-    btype  = (struct type *) xmalloc (sizeof (struct type));
+  if (p[-2] != '.')			/* ... */
+    argtypes[argcount] = builtin_type_void;	/* Ellist terminator */
   else
-    btype  = (struct type *) obstack_alloc (symbol_obstack,
-					    sizeof (struct type));
+    argtypes[argcount] = NULL;		/* List terminator */
 
-  if (main_type == 0)
-    {
-      main_type = btype;
-      bzero (btype, sizeof (struct type));
-      TYPE_MAIN_VARIANT (btype) = main_type;
-    }
-  else
-    {
-      bcopy (main_type, btype, sizeof (struct type));
-      TYPE_NEXT_VARIANT (main_type) = btype;
-    }
-/* TYPE_OFFSET (btype) = offset; */
-  if (via_public)
-    TYPE_FLAGS (btype) |= TYPE_FLAG_VIA_PUBLIC;
-  if (via_virtual)
-    TYPE_FLAGS (btype) |= TYPE_FLAG_VIA_VIRTUAL;
-  /* New type is permanent if type pointed to is permanent.  */
-  if (TYPE_FLAGS (type) & TYPE_FLAG_PERM)
-    TYPE_FLAGS (btype) |= TYPE_FLAG_PERM;
+  free (demangled_name);
 
-  /* In practice, this is never used.  */
-  TYPE_LENGTH (btype) = 1;
-  TYPE_CODE (btype) = TYPE_CODE_STRUCT;
+  f = TYPE_FN_FIELDLIST1 (type, i);
+  TYPE_FN_FIELD_PHYSNAME (f, j) = mangled_name;
 
-  return btype;
+  /* Now update the old "stub" type into a real type.  */
+  mtype = TYPE_FN_FIELD_TYPE (f, j);
+  TYPE_DOMAIN_TYPE (mtype) = type;
+  TYPE_ARG_TYPES (mtype) = argtypes;
+  TYPE_FLAGS (mtype) &= ~TYPE_FLAG_STUB;
 }
-#endif
 
 /* Given a type TYPE, return a type of functions that return that type.
    May need to construct such a type if this is the first use.  */
@@ -767,7 +720,11 @@ create_array_type (element_type, number)
 }
 
 
-/* Smash TYPE to be a type of members of DOMAIN with type TO_TYPE.  */
+/* Smash TYPE to be a type of members of DOMAIN with type TO_TYPE. 
+   A MEMBER is a wierd thing -- it amounts to a typed offset into
+   a struct, e.g. "an int at offset 8".  A MEMBER TYPE doesn't
+   include the offset (that's the value of the MEMBER itself), but does
+   include the structure type into which it points (for some reason).  */
 
 void
 smash_to_member_type (type, domain, to_type)
@@ -776,15 +733,12 @@ smash_to_member_type (type, domain, to_type)
   bzero (type, sizeof (struct type));
   TYPE_TARGET_TYPE (type) = to_type;
   TYPE_DOMAIN_TYPE (type) = domain;
-
-  /* In practice, this is never needed.  */
-  TYPE_LENGTH (type) = 1;
+  TYPE_LENGTH (type) = 1;	/* In practice, this is never needed.  */
   TYPE_CODE (type) = TYPE_CODE_MEMBER;
-
-  TYPE_MAIN_VARIANT (type) = lookup_member_type (domain, to_type);
 }
 
-/* Smash TYPE to be a type of method of DOMAIN with type TO_TYPE.  */
+/* Smash TYPE to be a type of method of DOMAIN with type TO_TYPE.
+   METHOD just means `function that gets an extra "this" argument'.  */
 
 void
 smash_to_method_type (type, domain, to_type, args)
@@ -794,12 +748,8 @@ smash_to_method_type (type, domain, to_type, args)
   TYPE_TARGET_TYPE (type) = to_type;
   TYPE_DOMAIN_TYPE (type) = domain;
   TYPE_ARG_TYPES (type) = args;
-
-  /* In practice, this is never needed.  */
-  TYPE_LENGTH (type) = 1;
+  TYPE_LENGTH (type) = 1;	/* In practice, this is never needed.  */
   TYPE_CODE (type) = TYPE_CODE_METHOD;
-
-  TYPE_MAIN_VARIANT (type) = lookup_method_type (domain, to_type, args);
 }
 
 /* Find which partial symtab on the partial_symtab_list contains
@@ -1039,7 +989,7 @@ lookup_symbol (name, block, namespace, is_a_field_of_this, symtab)
 	block = BLOCKVECTOR_BLOCK (bv, GLOBAL_BLOCK);
 	sym = lookup_block_symbol (block, name, namespace);
 	if (!sym)
-	  error ("Internal: global symbol found in psymtab but not in symtab");
+	  error ("Internal: global symbol `%s' found in psymtab but not in symtab", name);
 	if (symtab != NULL)
 	  *symtab = s;
 	return sym;
@@ -1071,7 +1021,7 @@ lookup_symbol (name, block, namespace, is_a_field_of_this, symtab)
 	block = BLOCKVECTOR_BLOCK (bv, STATIC_BLOCK);
 	sym = lookup_block_symbol (block, name, namespace);
 	if (!sym)
-	  error ("Internal: static symbol found in psymtab but not in symtab");
+	  error ("Internal: static symbol `%s' found in psymtab but not in symtab", name);
 	if (symtab != NULL)
 	  *symtab = s;
 	return sym;
@@ -1104,7 +1054,7 @@ lookup_symbol (name, block, namespace, is_a_field_of_this, symtab)
 	    block = BLOCKVECTOR_BLOCK (bv, STATIC_BLOCK);
 	    sym = lookup_demangled_block_symbol (block, name);
 	    if (!sym)
-	      error ("Internal: mangled static symbol found in psymtab but not in symtab");
+	      error ("Internal: mangled static symbol `%s' found in psymtab but not in symtab", name);
 	    if (symtab != NULL)
 	      *symtab = s;
 	    return sym;
@@ -1248,6 +1198,18 @@ lookup_partial_symbol (pst, name, global, namespace)
   return (struct partial_symbol *) 0;
 }
 
+/* Find the psymtab containing main(). */
+
+struct partial_symtab *
+find_main_psymtab ()
+{
+  register struct partial_symtab *pst;
+  for (pst = partial_symtab_list; pst; pst = pst->next)
+    if (lookup_partial_symbol (pst, "main", 1, VAR_NAMESPACE))
+	return pst;
+  return NULL;
+}
+
 /* Look for a symbol in block BLOCK.  */
 
 struct symbol *
@@ -1383,7 +1345,7 @@ find_pc_symtab (pc)
       ps = find_pc_psymtab (pc);
       if (ps && ps->readin)
 	printf_filtered (
-	  "(Internal error: pc in read in psymtab, but not in symtab.)\n");
+	  "(Internal error: pc 0x%x in read in psymtab, but not in symtab.)\n", pc);
 
       if (ps)
 	s = PSYMTAB_TO_SYMTAB (ps);
@@ -1467,6 +1429,8 @@ find_pc_line (pc, notcurrent)
     {
       /* Find the best line in this symtab.  */
       l = LINETABLE (s);
+      if (!l)
+        continue;
       len = l->nitems;
       prev_line = -1;
       first_line = -1;
@@ -1616,6 +1580,8 @@ find_line_common (l, lineno, exact_match)
 
   if (lineno <= 0)
     return -1;
+  if (l == 0)
+    return -1;
 
   len = l->nitems;
   for (i = 0; i < len; i++)
@@ -1904,7 +1870,6 @@ decode_line_1 (argptr, funfirstline, default_symtab, default_line)
     }
   while (p[0] == ' ' || p[0] == '\t') p++;
 
-  q = operator_chars (*argptr, &q1);
   if (p[0] == ':')
     {
 
@@ -1927,7 +1892,7 @@ decode_line_1 (argptr, funfirstline, default_symtab, default_line)
 				     (struct symtab **)NULL);
        
 	  if (sym_class &&
-	      (TYPE_CODE (SYMBOL_TYPE (sym_class)) == TYPE_CODE_STRUCT
+	      (   TYPE_CODE (SYMBOL_TYPE (sym_class)) == TYPE_CODE_STRUCT
 	       || TYPE_CODE (SYMBOL_TYPE (sym_class)) == TYPE_CODE_UNION))
 	    {
 	      /* Arg token is not digits => try it as a function name
@@ -2017,7 +1982,12 @@ decode_line_1 (argptr, funfirstline, default_symtab, default_line)
 		    }
 		  else
 		    tmp = copy;
-		  error ("that class does not have any method named %s", tmp);
+		  if (tmp[0] == '~')
+		    error ("The class `%s' does not have destructor defined",
+			   sym_class->name);
+		  else
+		    error ("The class %s does not have any method named %s",
+			   sym_class->name, tmp);
 		}
 	    }
 	  else
@@ -2030,21 +2000,9 @@ decode_line_1 (argptr, funfirstline, default_symtab, default_line)
       /* Extract the file name.  */
       p1 = p;
       while (p != *argptr && p[-1] == ' ') --p;
-      copy = (char *) alloca (p - *argptr + 1 + (q1 - q));
-      if (q1 - q)
-	{
-	  copy[0] = 'o';
-	  copy[1] = 'p';
-	  copy[2] = CPLUS_MARKER;
-	  bcopy (q, copy + 3, q1-q);
-	  copy[3 + (q1-q)] = 0;
-	  p = q1;
-	}
-      else
-	{
-	  bcopy (*argptr, copy, p - *argptr);
-	  copy[p - *argptr] = 0;
-	}
+      copy = (char *) alloca (p - *argptr + 1);
+      bcopy (*argptr, copy, p - *argptr);
+      copy[p - *argptr] = 0;
 
       /* Find that file's data.  */
       s = lookup_symtab (copy);
@@ -2156,7 +2114,16 @@ decode_line_1 (argptr, funfirstline, default_symtab, default_line)
 	  /* Convex: no need to suppress code on first line, if any */
 	  val.pc = pc;
 #else
-	  val.pc = (val.end && val.pc != pc) ? val.end : pc;
+	  /* If SKIP_PROLOGUE left us in mid-line, and the next line is still
+	     part of the same function:
+		advance to next line, 
+	        recalculate its line number (might not be N+1).  */
+	  if (val.pc != pc && val.end &&
+	      find_pc_misc_function (pc) == find_pc_misc_function (val.end)) {
+	    pc = val.end;	/* First pc of next line */
+	    val = find_pc_line (pc, 0);
+	  }
+	  val.pc = pc;
 #endif
 	  values.sals = (struct symtab_and_line *)xmalloc (sizeof (struct symtab_and_line));
 	  values.sals[0] = val;
@@ -2415,8 +2382,7 @@ sources_info ()
   
   if (symtab_list == 0 && partial_symtab_list == 0)
     {
-      printf (no_symtab_msg);
-      return;
+      error (no_symtab_msg);
     }
   
   printf_filtered ("Source files for which symbols have been read in:\n\n");
@@ -2710,7 +2676,6 @@ init_type (code, length, uns, name)
 
   type = (struct type *) xmalloc (sizeof (struct type));
   bzero (type, sizeof *type);
-  TYPE_MAIN_VARIANT (type) = type;
   TYPE_CODE (type) = code;
   TYPE_LENGTH (type) = length;
   TYPE_FLAGS (type) = uns ? TYPE_FLAG_UNSIGNED : 0;
@@ -2719,8 +2684,13 @@ init_type (code, length, uns, name)
   TYPE_NAME (type) = name;
 
   /* C++ fancies.  */
-  TYPE_NFN_FIELDS (type) = 0;
-  TYPE_N_BASECLASSES (type) = 0;
+  if (code == TYPE_CODE_STRUCT || code == TYPE_CODE_UNION)
+    {
+      TYPE_CPLUS_SPECIFIC (type)
+	= (struct cplus_struct_type *) xmalloc (sizeof (struct cplus_struct_type));
+      TYPE_NFN_FIELDS (type) = 0;
+      TYPE_N_BASECLASSES (type) = 0;
+    }
   return type;
 }
 
@@ -2945,7 +2915,7 @@ _initialize_symtab ()
      I also think "ptype" or "whatis" is more likely to be useful (but if
      there is much disagreement "info types" can be fixed).  */
   add_info ("types", types_info,
-	    "All types names, or those matching REGEXP.");
+	    "All type names, or those matching REGEXP.");
 
 #if 0
   add_info ("methods", methods_info,
