@@ -1,12 +1,26 @@
-/*
- * Ported by the State University of New York at Buffalo by the Distributed
- * Computer Systems Lab, Department of Computer Science, 1991.
- */
+/* Print instructions for Tahoe target machines, for GDB.
+   Copyright 1986, 1989, 1991 Free Software Foundation, Inc.
+   Contributed by the State University of New York at Buffalo, by the
+   Distributed Computer Systems Lab, Department of Computer Science, 1991.
+
+This file is part of GDB.
+
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program; if not, write to the Free Software
+Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #include <stdio.h>
-
 #include "defs.h"
-#include "param.h"
 #include "symtab.h"
 #include "tahoe-opcode.h"
 
@@ -98,7 +112,7 @@ print_insn_arg (d, p, addr, stream)
       case 0:
       case 1:
       case 2:
-      case 3:			/* Liter>al(short immediate byte) mode */
+      case 3:			/* Literal (short immediate byte) mode */
 	if (d[1] == 'd' || d[1] == 'f' || d[1] == 'g' || d[1] == 'h')
 	  {
 	    *(int *)&floatlitbuf = 0x4000 + ((p[-1] & 0x3f) << 4);
