@@ -427,7 +427,7 @@ long read_register_stack_integer ();
 /* Because INIT_FRAME_PC gets passed fromleaf, that's where we init
    not only ->pc and ->frame, but all the extra stuff, when called from
    get_prev_frame_info, that is.  */
-#define INIT_EXTRA_FRAME_INFO(fci) \
+#define INIT_EXTRA_FRAME_INFO(fromleaf, fci) \
   init_extra_frame_info(fci);
 void init_extra_frame_info ();
 #define INIT_FRAME_PC(fromleaf, fci) \
@@ -437,12 +437,8 @@ void init_frame_pc ();
 /* FRAME_CHAIN takes a FRAME
    and produces the frame's chain-pointer.
 
-   FRAME_CHAIN_COMBINE takes the chain pointer and the frame's nominal address
-   and produces the nominal address of the caller frame.
-
    However, if FRAME_CHAIN_VALID returns zero,
-   it means the given frame is the outermost one and has no caller.
-   In that case, FRAME_CHAIN_COMBINE is not used.  */
+   it means the given frame is the outermost one and has no caller.  */
 
 /* On the 29k, the nominal address of a frame is the address on the
    register stack of the return address (the one next to the incoming
@@ -465,8 +461,6 @@ void init_frame_pc ();
 #define FRAME_CHAIN_VALID(chain, thisframe) \
   (outside_startup_file (FRAME_SAVED_PC (thisframe)))
 
-#define FRAME_CHAIN_COMBINE(chain, thisframe) (0)
-
 /* Define other aspects of the stack frame.  */
 
 /* A macro that tells us whether the function invocation represented
